/*
   fmtcoma.cpp

   Copyright (C) 1996 Jim Howell
   The material in this document may be freely used by anyone
   for any purpose, private or commercial, so long as this
   copyright notice is included and all changes to anything in
   the document are listed.

   A program for testing the formatComma class.
*/


//  Turn off MSC warnings.
#if defined (_MSC_VER)
#if (_MSC_VER >=900)
#pragma warning (disable: 4699 4514)
#endif
#endif


#include <limits.h>
#include <locale.h>
#include <stdio.h>
#include <iostream.h>
#include <iomanip.h>

#include "fmtcoma.h"


/***************************************************************
   An array of test numbers that cover all critical
   posibilities.  Fortunately, only the number of digits in the
   number and its sign are important, so we can test all cases.
   The largest and smallest values are added to make sure it
   works at the extremes.
*/
static long tnum [] ={         1L,          -1L,
                              12L,         -12L,
                             123L,        -123L,
                            1234L,       -1234L,
                           12345L,      -12345L,
                          123456L,     -123456L,
                         1234567L,    -1234567L,
                        12345678L,   -12345678L,
                       123456789L,  -123456789L,
                      1234567890L, -1234567890L,
                         LONG_MAX,     LONG_MIN,
                      };


/***************************************************************
   This is a similar array for unsigned integers.
*/
static unsigned long unum []  ={         0L,
                                         1L,
                                        12L,
                                       123L,  
                                      1234L, 
                                     12345L,
                                    123456L,  
                                   1234567L, 
                                  12345678L,
                                 123456789L,  
                                1234567890L, 
                                  ULONG_MAX, 
                                };


/***************************************************************
   A class to put single quote marks around the output
   of formatComma.
*/
class frameNum {
public:
               frameNum    (const formatComma&  fc);
   const char* operator &  () const;
private:
   char        buff [35];
   };

inline frameNum::frameNum (const formatComma& fc)
{  sprintf (buff, "'%s'", &fc);  }

inline const char* frameNum::operator & () const
{  return buff;  }


/**************************************************************/
int main (
   int   ,
   char* [])
{
   setlocale (LC_ALL, "C");

   cout <<"Testing the signed integers." << endl;
   //  Test zero, of course.
   cout <<dec <<setw (11) <<0 <<"   "
        <<setw (19) <<&frameNum (formatComma (0L));
   cout <<endl;

   //  Test the contents of the array.
   for (int I =0; I <sizeof (tnum) /sizeof (tnum [0]); ++ I)   {
      cout <<dec <<setw (11) <<tnum [I] <<"   "
           <<setw (19) <<&frameNum (formatComma (tnum [I]));
      if ((I %2) ==0)
         cout <<"   |   ";
      else
         cout <<endl;
      }

   //  Now test the unsigned integers.
   cout <<endl <<"Testing the unsigned integers." << endl;
   for (int U =0; U <sizeof (unum) /sizeof (unum [0]); ++ U)   {
      cout <<dec <<setw (11) <<unum [U] <<"   "
           <<setw (19) <<&frameNum (formatComma (unum [U]))
           <<endl;
      }

   return 0;
   }

