/*
   fmtcoma.cpp

   Copyright (C) 1996 Jim Howell
   The material in this document may be freely used by anyone
   for any purpose, private or commercial, so long as this
   copyright notice is included and all changes to anything in
   the document are listed.

   A class for adding commas to a number converted
   to a string.
*/


/*
   Remove the comment on this #include statement if you are
   using MFC.  There is no pre-defined macro we can test.
*/
//#include "stdafx.h"


#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <iostream.h>

/*
   Find out what OS we are using.  Borland and Microsoft use
   different pre-defined macros.  Define one of
      FC_WIN32
      FC_WIN16
      FC_STD
*/

#if defined (__BORLANDC__)
//  Borland compiler.
#if defined (__WIN32__)
#define FC_WIN32
#elif defined (_Windows)
#define FC_WIN16
#else
#define FC_STD
#endif

#elif defined (_MSC_VER)
//  Microsoft compiler
#if defined (_WIN32)
#define FC_WIN32
#elif (_MSC_VER <900)
#define FC_WIN16
#else
#define FC_STD
#endif

#else
//  Figure out your own way of defining the macros.
#error Unknown comipiler.
#endif

#if defined (FC_WIN32) || defined (FC_WIN16)
#define FC_WIN
#endif


//  In MFC 4 this may not be included in STDAFX.
#if defined (FC_WIN32) && defined (_MFC_VER) && !defined (_WINNLS_)
#include <winnls.h>
#endif


//  Turn off MSVC warnings from a level 4 compile.
#if defined (_MSC_VER) && (_MSC_VER >=900)
#pragma warning (disable: 4699 4514 4201)
#endif

//  System-dependent headers.
#if defined (FC_WIN)
#include <windows.h>
#else
#include <locale.h>
#endif

/*
   Boolean definition.  This won't be needed once bool is a
   true keyword.  You may have to change this for your own
   system.
*/
#if defined (__BORLANDC__)
//  This is defined only if you include the Borland classlibs.
enum bool {false, true};
#else
#if !defined (BOOL)
#define BOOL   int
#endif
#if !defined (TRUE)
#define TRUE   1
#endif
#if !defined (FALSE)
#define FALSE  0
#endif

#define bool   BOOL
#define false  FALSE
#define true   TRUE
#endif

#include "fmtcoma.h"


/***************************************************************
   In outline this is what we do.  First, we convert the number
   to a string using standard C library calls.  This is put
   into a temporary "numerical buffer".  We copy this into the 
   "format buffer", going from the end to the beginning in both,
   putting in commas at every third point.  Then we add the
   sign, if there is one.  The numerical buffer is never changed.

   The helper class nBuf hides the details of the numerical
   buffer.  By using different constructors we also make it
   easy to call the appropriate conversion function.

   Before we copy this into the format buffer, we need to find
   the thousands separator.  How we do this depends on the
   operating system; the standard C locale is one way, 16-bit
   Windows another, and 32-bit Windows a third.  This handles
   all cases, and uses the standard comma as a default if it
   can't get it from the system.
*/


/***************************************************************
   Much of the code is inline, and so is written in bottom-up
   order.  The Borland compiler (at least up to 4.5) will not
   make functions with while inline, but it isn't a problem with
   the Microsoft compilers.  The following #define takes care of
   this.  This may be changed in version 5.
*/

#if defined (__BORLANDC__)
#define  b_inline
#else
#define  b_inline  inline
#endif


/***************************************************************
   This class handles the numerical buffer.
   
   The pointer pn is set to the last character in the string,
   and always points to the next character to be examined,
   except when we are done with the string, where it points
   before the first digit in the string.  This may be a sign
   character, or it may point in front of the buffer.

   The pointer pf points to the first digit in the string.  The
   first character may be a sign; we set the Boolean varaible
   is_signed to true if the number has a sign.

   This class isn't strictly necessary, as all these varaibles
   cound be declared in the constructor of formatComma.  Using
   this class makes the code there much clearer.  The inline
   code should obviate any inefficiencies.
*/
class nBuf {
public:
         nBuf     (long          n);
         nBuf     (unsigned long n);
   bool  Next     (char*&        pb);
   bool  HasNext  () const;
   bool  IsSigned () const;
   char  Sign     () const;
private:
   void  Init     ();
   char  buff [30];
   char* pf;
   char* pn;
   char* pd;
   bool  is_signed;
   };


/***************************************************************
   This is called from the constructors.
   
   Find the last character in the string pn.  Find the first
   digit in the string pf.

   Note:  this relies on ltoa() and ultoa() always returning a
   properly formatted string.
*/
inline void nBuf::Init ()
{
   pn =strchr (buff, '\0') -1;
   pf =buff;

   if (is_signed)
      ++ pf;
   }  //  nBuf::Init


/***************************************************************
   Convert the number to a string in buff.  We use separate
   constructors for signed and unsigned numbers because we call
   different formatting functions.
*/
inline nBuf::nBuf (
   long  n)
{
   ltoa (n, buff, 10);
   is_signed   =!isdigit (*buff);
   pd          =NULL;
   Init ();
   }  //  nBuf::nBuf


inline nBuf::nBuf (
   unsigned long  n)
{
   ultoa (n, buff, 10);
   is_signed   =false;
   pd          =NULL;
   Init ();
   }  //  nBuf::nBuf


/***************************************************************
   If a digit is available, put it in pb and return true.
   Otherwise, return false.
*/
inline bool nBuf::Next (
   char*&   pb)
{
   if (pn >=pf)  {
      *pb -- =*pn --;
      return true;
      }
   else
      return false;
   }  //  nBuf::Next


/***************************************************************
   Return whether there is another digit in the string.
*/
inline bool nBuf::HasNext () const
{
   return (pn >=pf);
   }  //  nBuf::HasNext


/***************************************************************
   Return whether the number is signed or not.
*/
inline bool nBuf::IsSigned () const
{
   return is_signed;
   }  //  nBuf::IsSigned


/***************************************************************
   Return the sign character.  This should be called only if the
   number does have a sign.
*/
inline char nBuf::Sign () const
{
   return *buff;
   }  //  nBuf::Sign


/***************************************************************
  We have to find the thousands separator for the user's
  environment.  The details depend upon the operating system.

  You might want to make some changes if you want to detect if
  the user changes the format separator while your program is
  running (which is possible in Windows).  If so, remove the
  static specifier on this method and the sep[] variable, along
  with the test to see of sep[] is already set.  That will slow
  things down slightly, of course.
  
  Alternatively, you could have your application watch for the
  WM_WININICHANGE message.  In that case, this would be better
  as a public method which is called somewhere during program
  initialization; you should then remove the call from the
  constructor.
*/
char formatComma::sep [4] ="";

inline void formatComma::SetSeparator ()
{
   if (!*sep)  {
#     if defined (FC_WIN32)
      GetLocaleInfo (GetUserDefaultLCID (), LOCALE_STHOUSAND,
         sep, sizeof (sep));
#     elif defined (FC_WIN16)
      //  16-bit Windows.  Read setting from WIN.INI.
      GetProfileString ("Intl", "sThousand", "", sep, sizeof (sep));
#     else
      //  Use the standard locale call.
      strcpy (sep, localeconv () ->thousands_sep);
#     endif
      if (!*sep)
         //  Sometimes it fails.  Maybe you would like an error
         //  message here.
         *sep =',';
      }
   }  //  formatComma::SetSeparator


/***************************************************************
   Copy the number from the numerical buffer to the format
   buffer, going backwards from the ends of each buffer.  Every
   third digit we add a comma to the format buffer, if we
   haven't reached the start of the number yet.

   For speed, we unroll the loop, and handle three digits each
   time through it.

   The pb pointer always points to the next place to put a digit,
   except at the end, where it points to the first digit in the
   formatted string.
*/
b_inline void formatComma::SetCommas (
   nBuf& nstr)
{
   pb    =buff +buff_len -1;
   *pb-- ='\0';
   SetSeparator ();

   while (true)   {
      if (!nstr.Next (pb))
         break;
      if (!nstr.Next (pb))
         break;
      if (!nstr.Next (pb))
         break;
      if (!nstr.HasNext ())
         break;
      *pb-- =*sep;
      }

   if (nstr.IsSigned ())
      *pb =nstr.Sign ();
   else
      ++pb;
   }  //  formatComma::formatComma


/***************************************************************
   The constructors use different versions of the numerical
   buffer, depending on its type.
*/
formatComma::formatComma (
   long  n)
{
   SetCommas (nBuf (n));
   }  //  formatComma::formatComma


formatComma::formatComma (
   unsigned long  n)
{
   SetCommas (nBuf (n));
   }  //  formatComma::formatComma

